# Voyager

Voyager is an NPM utility that helps observe elements and highlight the corresponding links when they intersect the viewport.

## Installation

```bash
npm install voyager

Usage
javascript
Copy code
import Voyager from 'voyager';

document.addEventListener('DOMContentLoaded', () => {
  const nav = new Voyager({
    selector: '[data-voyager]',
    activeClass: 'active',
    rootMarginTop: '-50px',
    rootMarginBottom: '-50px',
    threshold: 0.5
  });
});


To turn this script into an NPM utility, we can generalize it, modularize it, and make it more configurable for different projects. Here’s a breakdown of how we can accomplish this:

Steps to Convert the Script into an NPM Package
Structure the NPM Package: Organize the code into a format suitable for NPM, with a package.json, README, and a main file (index.js).
Make Configurations Flexible: Allow users to pass configurations easily and avoid hardcoded dependencies (e.g., references to Header).
Documentation: Add JSDoc comments for clear documentation of options and usage.
Export as a Module: Allow importing and initializing in any JavaScript environment with import or require.
Test Compatibility: Ensure compatibility with various JavaScript frameworks (like React or Vue) or vanilla JavaScript projects.
Here’s how the code would look after restructuring it as a utility:

1. Directory Structure
go
Copy code
my-voyager-package/
├── src/
│   └── voyager.js
├── index.js
├── package.json
└── README.md
2. src/voyager.js - Main Logic
javascript
Copy code
/**
 * Voyager - A utility to observe elements and highlight corresponding links
 * when they intersect the viewport.
 */

class Voyager {
	/**
	 * Creates a Voyager instance.
	 *
	 * @param {Object} options - Configuration options.
	 * @param {string} [options.selector = "[data-voyager]"] - Selector for links.
	 * @param {string} [options.activeClass = "active"] - Class to add when link's target is active.
	 * @param {string} [options.rootMarginTop = "0px"] - Top margin for intersection observer.
	 * @param {string} [options.rootMarginBottom = "0px"] - Bottom margin for intersection observer.
	 * @param {number} [options.threshold = 0.1] - Threshold for intersection observer.
	 */
	constructor({ selector = "[data-voyager]", activeClass = "active", rootMarginTop = "0px", rootMarginBottom = "0px", threshold = 0.1 } = {}) {
		this.selector = selector;
		this.activeClass = activeClass;
		this.rootMarginTop = rootMarginTop;
		this.rootMarginBottom = rootMarginBottom;
		this.threshold = threshold;

		this.refresh();
	}

	/**
	 * Refresh the intersection observer and re-initialize it.
	 */
	refresh() {
		this._links = document.querySelectorAll(`${this.selector} a`);

		if ("IntersectionObserver" in window && this._links.length > 0) {
			this.observer = new IntersectionObserver(this._handleIntersect.bind(this), {
				rootMargin: `${this.rootMarginTop} 0px ${this.rootMarginBottom} 0px`,
				threshold: this.threshold,
			});

			this._links.forEach((link) => {
				link.addEventListener("click", (event) => {
					event.preventDefault();
					const targetId = link.getAttribute("href");
					const targetElement = document.querySelector(targetId);
					targetElement?.scrollIntoView({ behavior: "smooth" });
				});

				const target = document.querySelector(link.hash);
				if (target) {
					this.observer.observe(target);
				}
			});
		}
	}

	/**
	 * Handles the intersection of observed elements.
	 *
	 * @param {Array} entries - Intersection observer entries.
	 */
	_handleIntersect(entries) {
		entries.forEach((entry) => {
			const id = `#${entry.target.getAttribute("id")}`;
			const anchor = document.querySelector(`[href="${id}"]`);

			if (entry.isIntersecting) {
				document.querySelectorAll(`.${this.activeClass}`).forEach((el) => el.classList.remove(this.activeClass));
				anchor?.classList.add(this.activeClass);
			}
		});
	}

	/**
	 * Destroys the observer and clears the properties.
	 */
	destroy() {
		if (this.observer) {
			this.observer.disconnect();
			this.observer = null;
		}
		this._links = null;
	}
}

export default Voyager;
3. index.js - Entry Point
javascript
Copy code
import Voyager from "./src/voyager.js";

export default Voyager;
4. package.json - NPM Package Metadata
json
Copy code
{
  "name": "voyager",
  "version": "1.0.0",
  "description": "An NPM utility to observe elements and highlight corresponding links when they intersect the viewport.",
  "main": "index.js",
  "scripts": {
    "test": "echo \"Error: no test specified\" && exit 1"
  },
  "keywords": ["voyager", "intersection observer", "scroll", "active link", "highlight links"],
  "author": "Your Name",
  "license": "MIT"
}
5. README.md - Documentation
markdown
Copy code
# Voyager

Voyager is an NPM utility that helps observe elements and highlight the corresponding links when they intersect the viewport.

## Installation

```bash
npm install voyager
```

## Usage
javascript
Copy code
import Voyager from 'voyager';

document.addEventListener('DOMContentLoaded', () => {
  const nav = new Voyager({
    selector: '[data-voyager]',
    activeClass: 'active',
    rootMarginTop: '-50px',
    rootMarginBottom: '-50px',
    threshold: 0.5
  });
});

## Configuration
Option	Type	Default	Description
selector	string	[data-voyager]	Selector for links
activeClass	string	active	Class to add when link's target is active
rootMarginTop	string	0px	Top margin for intersection observer
rootMarginBottom	string	0px	Bottom margin for intersection observer
threshold	number	0.1	Threshold for intersection observer

Methods
refresh(): Re-initialize the observer.
destroy(): Disconnect and clean up the observer.

License
MIT